<?php

defined( 'BASEPATH' ) OR exit( 'No direct script access allowed' );

/**
 * Support Helper
 *
 * @author Shahzaib
 */


/**
 * Get Support Model Object
 *
 * @return object
 */
if ( ! function_exists( 'get_sm_object' ) )
{
    function get_sm_object()
    {
        $ci =& get_instance();
        $ci->load->model( 'Support_model' );
        
        return $ci->Support_model;
    }
}

/**
 * Inform Department Users
 *
 * @param   object  $data
 * @param   integer $ticket_id
 * @return  mixed
 * @version 1.1
 */
if ( ! function_exists( 'inform_department_users' ) )
{
    function inform_department_users( $data, $ticket_id )
    {
        $ci =& get_instance();
        
        $location = "admin/tickets/ticket/{$ticket_id}";
        $status = true;
        
        if ( is_email_settings_filled() )
        {
            $hook = 'department_ticket';
            $template = $ci->Tool_model->email_template_by_hook_and_lang( $hook, config_item( 'language' ) );
            
            if ( empty( $template ) ) return 'missing_template';
            
            $subject = replace_placeholders( $template->subject, ['{DEPARTMENT_NAME}' => $data->name] );
            
            $for_message = [
                '{DEPARTMENT_NAME}' => $data->name,
                '{TICKET_URL}' => env_url( $location ),
                '{SITE_NAME}' => db_config( 'site_name' )
            ];
            
            $ci->load->library( 'ZMailer' );
        }
        
        $team = $data->team;
        $names = [];
        $emails = [];
        $ids = [];
        
        if ( $team !== 'all_users' )
        {
            $ci->load->model( 'User_model' );
            
            $users = json_decode( $team )->users;
            
            foreach ( $users as $user_id )
            {
                $user = $ci->User_model->get_by_id( $user_id );
                $names[] = $user->first_name . ' ' . $user->last_name;
                $emails[] = $user->email_address;
                $ids[] = $user->id;
            }
        }
        else
        {
            foreach ( get_team_users() as $user )
            {
                $names[] = $user->first_name . ' ' . $user->last_name;
                $emails[] = $user->email_address;
                $ids[] = $user->id;
            }
        }
        
        for ( $i = 0; $i < count( $emails ); $i++ )
        {
            send_notification( 'ticket_assigned_department', $location, $ids[$i], 1 );
            
            if ( is_email_settings_filled() && is_notifications_enabled( $ids[$i] ) )
            {
                $placeholders = $for_message;
                $placeholders['{USER_NAME}'] = $names[$i];
            
                $message = replace_placeholders( $template->template, $placeholders );
                $status = $ci->zmailer->send_email( $emails[$i], $subject, $message );
            }
        }
        
        return $status;
    }
}

/**
 * Send Reply Notification
 *
 * Use to send the ticket replied notification.
 *
 * @param  integer $user_id
 * @param  integer $ticket_id
 * @param  integer $reply_id
 * @param  string  $postfix
 * @return boolean
 */
if ( ! function_exists( 'send_reply_notification' ) )
{
    function send_reply_notification( $user_id, $ticket_id, $reply_id, $postfix = 'agent' )
    {
        $ci =& get_instance();
        $status = true;
        
        if ( ! in_array( $postfix, ['agent', 'user'] ) )
        {
            return false;
        }

        $ci->load->model( 'User_model' );

        $user = $ci->User_model->get_by_id( $user_id );

        if ( empty( $user ) ) return 'invalid_req';

        $area = ( $postfix === 'agent' ) ? 'user' : 'admin';
        $for_team_member = ( $postfix !== 'agent' ) ? 1 : 0;
        $location = "{$area}/support/ticket/{$ticket_id}?to_move_box=";
        $location .= md5( $reply_id );
        
        send_notification( "{$postfix}_replied_ticket", $location, $user_id, $for_team_member );
        
        if ( is_email_settings_filled() && is_notifications_enabled( $user_id ) )
        {
            $email_address = $user->email_address;
            $language = get_user_closer_language( $user->language );
            $hook = "ticket_replied_{$postfix}";
            $template = $ci->Tool_model->email_template_by_hook_and_lang( $hook, $language );

            if ( empty( $template ) ) return false;
            
            $subject = $template->subject;
            
            $message = replace_placeholders( $template->template, [
                '{USER_NAME}' => $user->first_name . ' ' . $user->last_name,
                '{TICKET_URL}' => env_url( $location ),
                '{SITE_NAME}' => db_config( 'site_name' )
            ]);
            
            $ci->load->library( 'ZMailer' );

            if ( ! $ci->zmailer->send_email( $email_address, $subject, $message ) )
            {
                $status = false;
            }
        }

        return $status;
    }
}

/**
 * Get FAQs Categories
 *
 * @param  string $order
 * @return object
 */
if ( ! function_exists( 'get_faqs_categories' ) )
{
    function get_faqs_categories( $order = 'DESC' )
    {
        return get_sm_object()->faqs_categories( $order );
    }
}

/**
 * Get FAQs Category Name
 *
 * @param  integer $id
 * @return string
 */
if ( ! function_exists( 'get_faqs_category_name' ) )
{
    function get_faqs_category_name( $id )
    {
        $category = get_sm_object()->faqs_category( $id );
        
        if ( empty( $category->name ) )
        {
            return '';
        }
        
        return $category->name;
    }
}

/**
 * Get FAQs by Category
 *
 * @param  integer $cat_id
 * @return object
 */
if ( ! function_exists( 'get_faqs_by_category' ) )
{
    function get_faqs_by_category( $cat_id )
    {
        return get_sm_object()->faqs_by_category( $cat_id );
    }
}

/**
 * Get Articles Categories
 *
 * @param  string $search
 * @param  string $order
 * @return object
 */
if ( ! function_exists( 'get_articles_categories' ) )
{
    function get_articles_categories( $search = 'main', $order = 'ASC' )
    {
        return get_sm_object()->articles_categories( $search, $order );
    }
}

/**
 * Get Articles Categories
 *
 * @param  integer $parent_id
 * @return object
 */
if ( ! function_exists( 'get_articles_subcategories' ) )
{
    function get_articles_subcategories( $parent_id )
    {
        return get_sm_object()->articles_subcategories( $parent_id );
    }
}

/**
 * Get Articles Category Data
 *
 * @param  integer $id
 * @param  string  $prop
 * @return mixed
 */
if ( ! function_exists( 'get_articles_category_data' ) )
{
    function get_articles_category_data( $id, $prop = '' )
    {
        $category = get_sm_object()->articles_category( $id );
        
        if ( empty( $prop ) ) return $category;
        
        return $category->{$prop};
    }
}

/**
 * Get Articles by Category
 *
 * @param  integer $id
 * @param  boolean $count
 * @return object
 */
if ( ! function_exists( 'get_articles_by_category' ) )
{
    function get_articles_by_category( $id, $count = false )
    {
        return get_sm_object()->articles_by_category( $id, $count );
    }
}

/**
 * Log Ticket Activity
 *
 * @param  string  $key Language key without "av_" prefix.
 * @param  integer $id
 * @param  integer $user_id Agent user ID
 * @return void
 */
if ( ! function_exists( 'log_ticket_activity' ) )
{
    function log_ticket_activity( $key, $id, $user_id = 0 )
    {
        $data = [
            'message_key' => 'av_' . $key,
            'ticket_id' => $id,
            'created_at' => time()
        ];
        
        if ( ! empty( $user_id ) )
        {
            $data['user_id'] = $user_id;
        }
        
        get_sm_object()->log_ticket_activity( $data );
    }
}

/**
 * Manage Ticket Sub Status
 *
 * @param  integer $status
 * @param  string  $area
 * @return string
 */
if ( ! function_exists( 'manage_ticket_sub_status' ) )
{
    function manage_ticket_sub_status( $status, $area = 'user' )
    {
        $string = '';
        
        if ( $status == 1 )
        {
            if ( $area === 'user' ) $string = lang( 'awaiting_reply' );
            else $string = lang( 'unanswered' );
        }
        else if ( $status == 2 )
        {
            if ( $area === 'user' ) $string = lang( 'support_replied' );
            else $string = lang( 'replied' );
        }
        else if ( $status == 3 )
        {
            $string = lang( 'solved' );
        }
        
        return $string;
    }
}

/**
 * Ticket Sub Status Color ( Label )
 *
 * @param  integer $status
 * @param  string  $area
 * @return string
 */
if ( ! function_exists( 'ticket_sub_status_color' ) )
{
  function ticket_sub_status_color( $status, $area = 'user' )
  {
      $string = '';
      
      if ( $status == 1 )
      {
          if ( $area === 'user' ) $string = 'bg-secondary';
          else $string = 'badge-danger';
      }
      else if ( $status == 2 )
      {
          if ( $area === 'user' ) $string = 'bg-info';
          else $string = 'badge-info';
      }
      else if ( $status == 3 )
      {
          if ( $area === 'user' ) $string = 'bg-success';
          else $string = 'badge-success';
      }
      
      return $string;
  }
}

/**
 * Manage Ticket Sub Status
 *
 * @param  integer $status
 * @return string
 */
if ( ! function_exists( 'manage_ticket_status' ) )
{
    function manage_ticket_status( $status )
    {
        $string = '';
        
        if ( $status == 1 )
        {
            $string = lang( 'opened' );
        }
        else
        {
            $string = lang( 'closed' );
        }
        
        return $string;
    }
}

/**
 * Ticket Status Color ( Label )
 *
 * @param  integer $status
 * @param  string  $area
 * @return string
 */
if ( ! function_exists( 'ticket_status_color' ) )
{
    function ticket_status_color( $status, $area = 'user' )
    {
        $string = '';
        
        if ( $status == 1 )
        {
            if ( $area === 'user' ) $string = 'bg-primary';
            else $string = 'badge-primary';
        }
        else
        {
            if ( $area === 'user' ) $string = 'bg-danger';
            else $string = 'badge-danger';
        }
        
        return $string;
    }
}

/**
 * Ticket Priority Color ( Label )
 *
 * @param  string $priority
 * @param  string $area
 * @return string
 */
if ( ! function_exists( 'ticket_priority_color' ) )
{
    function ticket_priority_color( $priority, $area = 'user' )
    {
        $string = '';
        
        if ( $priority == 'low' )
        {
            if ( $area === 'user' ) $string = 'bg-primary';
            else $string = 'badge-primary';
        }
        else if ( $priority == 'medium' )
        {
            if ( $area === 'user' ) $string = 'bg-warning';
            else $string = 'badge-warning';
        }
        else
        {
            if ( $area === 'user' ) $string = 'bg-danger';
            else $string = 'badge-danger';
        }
        
        return $string;
    }
}

/**
 * Get Articles Category Data
 *
 * @param  integer $ticket_id
 * @return integer
 */
if ( ! function_exists( 'ticket_replies_count' ) )
{
    function ticket_replies_count( $ticket_id )
    {
        return get_sm_object()->ticket_replies_count( $ticket_id );
    }
}
