<?php defined( 'BASEPATH' ) OR exit( 'No direct script access allowed' ); ?>
<div class="content">
  <div class="container-fluid">
    <div class="row">
      <div class="col-sm-12">
        <div class="card">
          <div class="card-header d-flex align-items-center">
            <h3 class="card-title"><?php echo html_escape( $card_title ); ?></h3>
            
            <?php if ( $assigned === true || ! $this->zuser->has_permission( 'all_tickets' ) ) { ?>
              <p class="ml-2 text-muted float-left text-sm"><?php echo lang( 'assigned_tickets_msg' ); ?></p>
            <?php } ?>
            
            <?php if ( $this->zuser->has_permission( 'all_tickets' ) ) { ?>
              <div class="card-tools ml-auto">
                <a href="<?php echo env_url( 'admin/tickets/create_ticket' ); ?>" class="btn btn-success text-sm">
                  <i class="fas fa-plus-circle mr-2"></i> <?php echo lang( 'create_ticket' ); ?>
                </a>
              </div>
              <!-- /.card-tools -->
            <?php } ?>
            
          </div>
          <!-- /.card-header -->
          <div class="card-body pt-0 pb-0 records-card-body">
            <div class="table-responsive">
              <table class="custom-table table z-table table-striped text-nowrap table-valign-middle mb-0">
                <thead class="records-thead">
                  <tr>
                    <th class="th-2"><?php echo lang( 'id' ); ?></th>
                    <th><?php echo lang( 'subject' ); ?></th>
                    <th class="text-right"><?php echo lang( 'reply_status' ); ?></th>
                    <th class="text-right th-2"><?php echo lang( 'status' ); ?></th>
                    <th class="text-right th-2"><?php echo lang( 'priority' ); ?></th>
                    <th class="text-right th-2"><?php echo lang( 'updated' ); ?></th>
                    <th class="text-right th-2"><?php echo lang( 'created' ); ?></th>
                    <th class="text-right th-2"><?php echo lang( 'actions' ); ?></th>
                  </tr>
                </thead>
                <tbody class="records-tbody text-sm">
                  <?php
                  if ( ! empty( $tickets ) )
                  {
                    foreach ( $tickets as $ticket ) { ?>
                      <tr id="record-<?php echo html_escape( $ticket->id ); ?>">
                        <td><?php echo html_escape( $ticket->id );?></td>
                        <td>
                          <span class="mr-1"><?php echo html_escape( $ticket->subject );?></span>
                          
                          <?php if ( $ticket->is_read_assigned == 0 && ( $ticket->assigned_to == $this->zuser->get( 'id' ) && ( $assigned === true || ! $this->zuser->has_permission( 'all_tickets' ) ) ) ) { ?>
                            <span class="badge badge-danger"><?php echo lang( 'newly_assigned' ); ?></span>
                          <?php } ?>
                          
                          <?php if ( $ticket->is_read == 0 && $ticket->sub_status == 1 && ( $ticket->assigned_to == null || $ticket->assigned_to == $this->zuser->get( 'id' ) ) ) { ?>
                            <span class="badge badge-danger"><?php echo lang( 'unread' ); ?></span>
                          <?php } ?>
                        </td>
                        <td class="text-right">
                          <span class="badge <?php echo ticket_sub_status_color( $ticket->sub_status, 'admin' ); ?>">
                            <?php echo manage_ticket_sub_status( $ticket->sub_status, 'admin' ); ?>
                          </span>
                        </td>
                        <td class="text-right">
                          <span class="badge <?php echo ticket_status_color( $ticket->status, 'admin' ); ?>">
                            <?php echo manage_ticket_status( $ticket->status );?>
                          </span>
                        </td>
                        <td class="text-right">
                          <span class="badge <?php echo ticket_priority_color( $ticket->priority, 'admin' ); ?>"><?php echo lang( html_escape( $ticket->priority ) ); ?></span>
                        </td>
                        <td class="text-right"><?php manage_updated_at( html_escape( $ticket->updated_at ) ); ?></td>
                        <td class="text-right"><?php echo get_date_time_by_timezone( html_escape( $ticket->created_at ) ); ?></td>
                        <td class="text-right">
                          <div class="btn-group">
                            <a href="<?php echo env_url( 'admin/tickets/ticket/' . html_escape( $ticket->id ) ); ?>" class="ml-2 btn btn-primary btn-sm">
                              <i class="fas fa-eye"></i>
                            </a>
                            <button class="btn btn-sm btn-danger tool" data-id="<?php echo html_escape( $ticket->id ); ?>" data-toggle="modal" data-target="#delete">
                              <i class="fas fa-trash tool-c"></i>
                            </button>
                          </div>
                          <!-- /.btn-group -->
                        </td>
                      </tr>
                      <?php }
                    } else {
                  ?>
                    <tr id="record-0">
                      <td colspan="8"><?php echo lang( 'no_records_found' ); ?></td>
                    </tr>
                  <?php } ?>
                </tbody>
              </table>
            </div>
            <!-- /.table-responsive -->
            
            <div class="clearfix"><?php echo $pagination; ?></div>
            
          </div>
          <!-- /.card-body -->
        </div>
        <!-- /.card -->
      </div>
      <!-- /.col -->
    </div>
    <!-- /.row -->
  </div>
  <!-- /.container-fluid -->
</div>
<!-- /.content -->

<?php load_modals( 'delete' ); ?>