<?php

defined( 'BASEPATH' ) OR exit( 'No direct script access allowed' );

/**
 * Users Controller ( Admin )
 *
 * @author Shahzaib
 */
class Users extends MY_Controller {

    /**
     * Class Constructor
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
        
        if ( ! $this->zuser->is_logged_in ) env_redirect( 'login' );

        check_page_authorization( 'users' );
        
        $this->sub_area = 'users';
        $this->area = 'admin';
        
        $this->load->model( 'User_model' );
        $this->load->library( 'pagination' );
    }
    
    /**
     * New User Page
     *
     * @return void
     */
    public function new_user()
    {
        $this->set_admin_reference( 'others' );
        
        $data['data']['roles'] = $this->Setting_model->roles();
        $data['title'] = sub_title( lang( 'users' ), lang( 'new_user' ) );
        $data['view'] = 'new_user';
        
        $this->load_panel_template( $data );
    }
    
    /**
     * Users Invites Page
     *
     * @return void
     */
    public function invites()
    {
        $this->set_admin_reference( 'others' );
        
        $config['base_url'] = env_url( 'admin/users/invites' );
        $config['total_rows'] = $this->User_model->invites( ['count' => true] );
        $config['per_page'] = PER_PAGE_RESULTS_PANEL;
        $offset = get_offset( $config['per_page'] );
        
        $this->pagination->initialize( $config );
        
        $data['data']['pagination'] = $this->pagination->create_links();
        
        $invites = $this->User_model->invites([
            'limit' => $config['per_page'],
            'offset' => $offset
        ]);
        
        $data['data']['invites'] = $invites;
        $data['delete_method'] = 'delete_invitation';
        $data['title'] = sub_title( lang( 'users' ), lang( 'invites' ) );
        $data['view'] = 'invites';
        
        $this->load_panel_template( $data );
    }
    
    /**
     * User Sent Emails Page
     *
     * @param  string $username
     * @return void
     */
    public function sent_emails( $username = '' )
    {
        $this->set_admin_reference( 'others' );
        
        $username = do_secure_l( $username );
        $user = $this->User_model->get_by_username( $username );
        
        if ( empty( $user ) ) env_redirect( 'admin/users/manage' );
        
        $config['base_url'] = env_url( "admin/users/sent_emails/{$username}" );
        
        $config['total_rows'] = $this->User_model->sent_emails([
            'user_id' => $user->id,
            'count' => true
        ]);
        
        $config['per_page'] = PER_PAGE_RESULTS_PANEL;
        $offset = get_offset( $config['per_page'], 5 );
        
        $this->pagination->initialize( $config );
        $data['data']['pagination'] = $this->pagination->create_links();
        
        $sent_emails = $this->User_model->sent_emails([
            'user_id' => $user->id,
            'limit' => $config['per_page'],
            'offset' => $offset
        ]);

        $data['data']['sent_emails'] = $sent_emails;
        $data['data']['user_email_address'] = $user->email_address;
        $data['delete_method'] = 'delete_sent_email';
        $data['title'] = sub_title( lang( 'users' ), lang( 'sent_emails' ) );
        $data['view'] = 'sent_emails';
        
        $this->load_panel_template( $data );
    }
    
    /**
     * User Sessions Page
     *
     * @param  string $username
     * @return void
     */
    public function sessions( $username = '' )
    {
        $this->set_admin_reference( 'others' );
        
        $username = do_secure_l( $username );
        $user = $this->User_model->get_by_username( $username );
        
        if ( empty( $user ) ) env_redirect( 'admin/users/manage' );
        
        $config['base_url'] = env_url( "admin/users/sessions/{$username}" );
        
        $config['total_rows'] = $this->Tool_model->user_sessions([
            'user_id' => $user->id,
            'count' => true
        ]);
        
        $config['per_page'] = PER_PAGE_RESULTS_PANEL;
        $offset = get_offset( $config['per_page'], 5 );
        
        $this->pagination->initialize( $config );
        $data['data']['pagination'] = $this->pagination->create_links();
        
        $sessions = $this->Tool_model->user_sessions([
            'user_id' => $user->id,
            'limit' => $config['per_page'],
            'offset' => $offset
        ]);
        
        $data['data']['sessions'] = $sessions;
        $data['data']['user_email_address'] = $user->email_address;
        $data['data']['username'] = $username;
        $data['title'] = sub_title( lang( 'users' ), lang( 'sessions' ) );
        $data['view'] = 'sessions';
        
        $this->load_panel_template( $data );
    }
    
    /**
     * Manage Users Page
     *
     * @return void
     */
    public function manage()
    {
        $this->set_admin_reference( 'others' );
        
        $searched = do_secure( get( 'search' ) );
        $filter = do_secure( get( 'filter' ) );
        $role = intval( get( 'role' ) );
        $config['base_url'] = env_url( 'admin/users/manage' );
        
        $config['total_rows'] = $this->User_model->users([
            'filter' => $filter,
            'searched' => $searched,
            'role' => $role,
            'count' => true
        ]);
        
        $config['per_page'] = PER_PAGE_RESULTS_PANEL;
        $offset = get_offset( $config['per_page'] );
        
        $this->pagination->initialize( $config );
        $data['data']['pagination'] = $this->pagination->create_links();
        
        $data['data']['users'] = $this->User_model->users([
            'filter' => $filter,
            'searched' => $searched,
            'role' => $role,
            'limit' => $config['per_page'],
            'offset' => $offset
        ]);
        
        $data['data']['roles'] = $this->Setting_model->roles();
        $data['delete_method'] = 'delete_user';
        $data['title'] = sub_title( lang( 'users' ), lang( 'manage' ) );
        $data['view'] = 'manage';
        
        $this->load_panel_template( $data );
    }
    
    /**
     * Edit User Page
     *
     * @param  integer $user_id
     * @return void
     */
    public function edit_user( $user_id = 0 )
    {
        $user_id = intval( $user_id );
        $user = $this->User_model->get_by_id( $user_id );
        
        if ( empty( $user ) ) env_redirect( 'admin/users/manage' );
        
        $this->set_admin_reference( 'others' );
        
        $data['title'] = sub_title( lang( 'users' ), lang( 'edit_user' ) );
        $data['data']['roles'] = $this->Setting_model->roles();
        $data['data']['user'] = $user;
        $data['view'] = 'edit_user';
        
        $this->load_panel_template( $data );
    }
}
